#ifndef ACTIVTIY_H
#define ACTIVTIY_H

#include "Monitor.h"
#include "Action.h"
#include "ActivityInstance.h"

<namespace>{

class Activity{
private:
	ActionContainer *first,*last;
public:
	Activity():first(new ActionContainer()),last(NULL){
		first->action=NULL;
		last=first;
	}
	virtual ~Activity(){
		ActionContainer *h1=first,*h2;
		while(h1!=NULL){
			h2=h1;
			h1=h1->next;
			delete h2;
		}
	}

    /**
	 * Adds a whole activity to the action. An activity cannot be removed as a whole,
	 */
	void addAction(Activity* a){
		if(a==NULL){return;}
		if(first->action==NULL){
			first=a->first;
		}else{
			last->next=a->first;
		}
	}

	/**
	 * Adds and returns a new interruptible action that sleeps for a given time.
	 */
	Action* addInterrupttibleSleep(unsigned long sl){
		if(sl==0){return NULL;}
		Action *a=new Action();
		if(first->action==NULL){
			first->action=a;
			first->wait=sl;
		}else{
			last->next=new ActionContainer();
			last=last->next;
			last->action=a;
			last->wait=sl;
		}
		return a;
	}

   /**
	 * Returns a new executable instance of this activity.
	 */
	ActivityInstance* getInstance(){
		return new ActivityInstance(first);
	}

    /**
     * Adds a new action to the activity.
     */
    void addAction(Action* a){
        if(a==NULL){return;}
        if(first->action==NULL){
            first->action=a;
            first->wait=0;
        }else{
            last->next=new ActionContainer();
            last=last->next;
            last->action=a;
            last->wait=0;
        }
    }

    /**
     * Removes all actions from the activity that are the same as the given action.
     */
    void removeAction(Action* g){
            ActionContainer *k=first->next,*j=first;
            if(first->action==NULL){
                    return;
            }else if(first->action==g){
                    if(first->next!=NULL){
                            first=first->next;
                    }else{
                            first->action=NULL;
                    }
            }
            while(k!=NULL && k!=first){
                    if(k->action==g){
                            j->next=k->next;
                            delete(k);
                    }else{
                            j=k;
                    }
                    k=k->next;
            }
    }

    /**
     * Executes the activity.
     */
    virtual void execute(){
        if(first->action==NULL){
            return;
        }
        ActionContainer *j=first;
        while(j!=NULL){
            j->action->execute();
            j=j->next;
        }
    }
};
}

#endif /* ACTIVTIY*/
