#ifndef ACTIVITYINSTANCE_H
#define ACTIVITYINSTANCE_H

#include "Thread.h"
#include "Action.h"
#include "Monitor.h"

<namespace>{

struct ActionContainer{
	Action* action;
	unsigned long wait;
	ActionContainer* next;
};

class ActivityInstance{

private:
	Thread *t;
	Monitor end;
	bool halt=true,run=false;
	ActionContainer* first;

private:
	static void* thread(void* h){
		THREAD *t=(THREAD*)h;
		ActivityInstance *ai=(ActivityInstance*)t->ptr;
		ActionContainer *current=ai->first;
		if(current->action!=NULL){
			while(current!=NULL){
				if(ai->halt){
					break;
				}
				current->action->execute();
				if(current->wait>0){
					t->thread->threadSleep(current->wait);
				}
				current=current->next;
			}
		}
		ai->endrun();
		return NULL;
	}

public:
	ActivityInstance(ActionContainer* a):t(new Thread(thread,this)),first(a){
	}
	~ActivityInstance(){
		delete t;
	}

public:
	bool runs(){
		return run;
	}

	/**
	 * Waits until the activity ended.
	 */
	void await(){
		end.lock();
		end.unlock();
	}

private:
	void endrun(){
		run=false;
		end.unlock();
	}

public:
	/**
	 * Stops the activity.
	 */
	void stop(){
		halt=true;
		if(t!=NULL){
			t->interruptThread();
		}
	}

public:
	/**
	 * Starts the activity in a separate thread.
	 */
	void start(){
		end.lock();
		halt=false;
		run=true;
		t->startThread();
	}
};
}

#endif //ACTIVITYINSTANCE_H
