#ifndef MONITOR
#define MONITOR

#if defined _WIN64 || defined _WIN32
#include<windows.h>
<namespace>{

class Monitor{
private:
	HANDLE handle;

public:
	Monitor():handle(CreateSemaphore(NULL,1,1,NULL)){
	};
	~Monitor(){
		CloseHandle(handle);
	}

	/**
	 * Locks the state for a thread. Returns false if procedure was successful.
	 */
	bool lock(){
		DWORD d=WaitForSingleObject(handle,INFINITE);
		return d==WAIT_OBJECT_0;
	}

	/**
	 * Releases lock on state for a thread. Returns false if procedure was successful.
	 */
	bool unlock(){
		return ReleaseSemaphore(handle,1,NULL);
	}
};
}

#else
#include<semaphore.h>
<namespace>{

class Monitor{
private:
	sem_t sem;

public:
	Monitor(){
		sem_init(&sem,0,1);
	};
	~Monitor(){
		sem_close(&sem);
	}

	/**
	 * Locks the state for a thread. Returns false if procedure was successful.
	 */
	bool lock(){
		return sem_wait(&sem);
	}

	/**
	 * Releases lock on state for a thread. Returns false if procedure was successful.
	 */
	bool unlock(){
		return sem_post(&sem);
	}
};
}

#endif

#endif //MONITOR
