#ifndef OBJECT_H
#define OBJECT_H

#include<string>

<namespace>{

class Object{
public:
	int ref;

	static const int POINTER=0,
					 STRING=1,
					 CHARACTER=2,
					 SHORT=3,
					 INTEGER=4,
					 FLOAT=5,
					 LONG=6,
					 DOUBLE=7,
					 BOOLEAN=8;

	Object(){
		ref=0;
	}

	virtual ~Object(){
	}

	virtual int getType()=0;
	
	virtual bool equals(Object* o){
		return o==this;
	}
};

template<typename T,int type>
class Atom : public Object{
protected:
	T value;
public:
	Atom(T val):value(val){
	}

	virtual ~Atom(){
	}

	T getValue(){
		return value;
	}

	int getType(){
		return type;
	}
	
	virtual bool equals(Object* o){
		if(o!=NULL && o->getType()==getType()){
			Atom<T,type>* other=static_cast<Atom<T,type>*>(o);
			return other->getValue()==getValue();
		}else{
			return Object::equals(o);
		}
	}
};

class Float:public Atom<float,Object::FLOAT>{
public:
	Float(float i):Atom(i){
	}

	virtual ~Float(){
	}
};

class Short :public Atom<short, Object::SHORT>{
public:
	Short(short i):Atom(i){
	}

	virtual ~Short(){
	}
};

class Integer :public Atom<int, Object::INTEGER>{
public:
	Integer(int i):Atom(i){
	}

	virtual ~Integer(){
	}
};

class Character :public Atom<char, Object::CHARACTER>{
public:
	Character(char i):Atom(i){
	}

	virtual ~Character(){
	}
};

class Long :public Atom<long, Object::LONG>{
public:
	Long(long i):Atom(i){
	}

	virtual ~Long(){
	}
};

class Double :public Atom<double, Object::DOUBLE>{
public:
	Double(double i):Atom(i){
	}

	virtual ~Double(){
	}
};

class Boolean :public Atom<bool, Object::BOOLEAN>{
public:
	Boolean(bool i):Atom(i){
	}

	virtual ~Boolean(){
	}
};

class String:public Object{
private:
	std::string value;
public:
	String(std::string i):value(i){
	}

	virtual ~String(){
	}

	int getType(){
		return STRING;
	}

	std::string getValue(){
		return value;
	}
};

class Pointer:public Atom<void*,Object::POINTER>{
public:
	Pointer(void* i):Atom(i){
	}

	virtual ~Pointer(){
	}
};
}

#endif /*OBJECT_H*/
