#ifndef OPERATIONS_H_
#define OPERATIONS_H_

#include "Object.h"
#include<iostream>
#include<string>

<namespace>{


/**
 * Implements all logic and arithmetic operations.
 */
class Operations{

private:
	static const char *const INTEGER,
					  *const SHORT,
					  *const STRING,
					  *const POINTER,
					  *const DOUBLE,
					  *const LONG,
					  *const CHARACTER,
					  *const BOOL,
					  *const FLOAT;

	static void errorMessage(std::string* op,Object* a,Object* b){
		std::cerr<<"Operator \""<<op<<"\" not applicable to "<<getType(a->getType())<<" and "<<getType(b->getType())<<std::endl;
	}

public:
	static inline const char* getType(int type){
		if(type==Object::BOOLEAN){
			return BOOL;
		}else if(type==Object::FLOAT){
			return FLOAT;
		}else if(type==Object::STRING){
			return STRING;
		}else if(type==Object::DOUBLE){
			return DOUBLE;
		}else if(type==Object::INTEGER){
			return INTEGER;
		}else if(type==Object::LONG){
			return LONG;
		}else if(type==Object::SHORT){
			return SHORT;
		}else if(type==Object::CHARACTER){
			return CHARACTER;
		}else{
			return POINTER;
		}
	}

public:
	static void deleteObject(Object* h){
		if(h==NULL){return;}
		if(h->ref==0){
			delete h;
		}
	}

private:

	static inline Object* arithmeticAdditionH(Object* a,Object* b){
		if(a->getType()==Object::SHORT){
			Short* g=(Short*)a;
		    if(b->getType()==Object::SHORT){
		    	Short* h=(Short*)b;
		    	return new Short(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::INTEGER){
		    	Integer* h=(Integer*)b;
		    	return new Integer(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::FLOAT){
		    	Float* h=(Float*)b;
		    	return new Float(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::LONG){
		    	Long* h=(Long*)b;
		    	return new Long(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::DOUBLE){
		    	Double* h=(Double*)b;
		    	return new Float(h->getValue()+g->getValue());
		    }else{
		    	std::string s("+");
		    	errorMessage(&s,a,b);
		    }
		}else if(a->getType()==Object::INTEGER){
			Integer* g=(Integer*)a;
		    if(b->getType()==Object::SHORT){
		    	Short* h=(Short*)b;
		    	return new Integer(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::INTEGER){
		    	Integer* h=(Integer*)b;
		    	return new Integer(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::FLOAT){
		    	Float* h=(Float*)b;
		    	return new Float(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::LONG){
		    	Long* h=(Long*)b;
		    	return new Long(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::DOUBLE){
		    	Double* h=(Double*)b;
		    	return new Double(h->getValue()+g->getValue());
		    }else{
		    	std::string s("+");
		        errorMessage(&s,a,b);
		    }
		}else if(a->getType()==Object::DOUBLE){
			Double* g=(Double*)a;
		    if(b->getType()==Object::SHORT){
		    	Short* h=(Short*)b;
		    	return new Double(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::INTEGER){
		    	Integer* h=(Integer*)b;
		    	return new Double(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::FLOAT){
		    	Float* h=(Float*)b;
		    	return new Double(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::LONG){
		    	Long* h=(Long*)b;
		    	return new Double(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::DOUBLE){
		    	Double* h=(Double*)b;
		    	return new Double(h->getValue()+g->getValue());
		    }else{
		    	std::string s("+");
		    	errorMessage(&s,a,b);
		    }
		}else if(a->getType()==Object::LONG){
			Long* g=(Long*)a;
		    if(b->getType()==Object::SHORT){
		    	Short* h=(Short*)b;
		    	return new Long(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::INTEGER){
		    	Integer* h=(Integer*)b;
		    	return new Long(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::FLOAT){
		    	Float* h=(Float*)b;
		    	return new Double(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::LONG){
		    	Long* h=(Long*)b;
		    	return new Long(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::DOUBLE){
		    	Double* h=(Double*)b;
		    	return new Long(h->getValue()+g->getValue());
		    }else{
		    	std::string s("+");
		    	errorMessage(&s,a,b);
		    }
		}else if(a->getType()==Object::FLOAT){
			Float* g=(Float*)a;
		    if(b->getType()==Object::SHORT){
		    	Short* h=(Short*)b;
		    	return new Float(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::INTEGER){
		    	Integer* h=(Integer*)b;
		    	return new Float(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::FLOAT){
		    	Float* h=(Float*)b;
		    	return new Float(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::LONG){
		    	Long* h=(Long*)b;
		    	return new Double(h->getValue()+g->getValue());
		    }else if(b->getType()==Object::DOUBLE){
		    	Double* h=(Double*)b;
		    	return new Double(h->getValue()+g->getValue());
		    }else{
		    	std::string s("+");
		    	errorMessage(&s,a,b);
		    }
		}else{
			std::string s("+");
			errorMessage(&s,a,b);
		}
		return NULL;
	}

	static inline Object* arithmeticSubtractionH(Object* a,Object* b){
		if(a->getType()==Object::SHORT){
			Short* g=(Short*)a;
		    if(b->getType()==Object::SHORT){
		    	Short* h=(Short*)b;
		    	return new Short(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::INTEGER){
		    	Integer* h=(Integer*)b;
		    	return new Integer(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::FLOAT){
		    	Float* h=(Float*)b;
		    	return new Float(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::LONG){
		    	Long* h=(Long*)b;
		    	return new Long(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::DOUBLE){
		    	Double* h=(Double*)b;
		    	return new Float(h->getValue()-g->getValue());
		    }else{
		    	std::string s("-");
		    	errorMessage(&s,a,b);
		    }
		}else if(a->getType()==Object::INTEGER){
			Integer* g=(Integer*)a;
		    if(b->getType()==Object::SHORT){
		    	Short* h=(Short*)b;
		    	return new Integer(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::INTEGER){
		    	Integer* h=(Integer*)b;
		    	return new Integer(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::FLOAT){
		    	Float* h=(Float*)b;
		    	return new Float(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::LONG){
		    	Long* h=(Long*)b;
		    	return new Long(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::DOUBLE){
		    	Double* h=(Double*)b;
		    	return new Double(h->getValue()-g->getValue());
		    }else{
		    	std::string s("-");
		    	errorMessage(&s,a,b);
		    }
		}else if(a->getType()==Object::DOUBLE){
			Double* g=(Double*)a;
		    if(b->getType()==Object::SHORT){
		    	Short* h=(Short*)b;
		    	return new Double(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::INTEGER){
		    	Integer* h=(Integer*)b;
		    	return new Double(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::FLOAT){
		    	Float* h=(Float*)b;
		    	return new Double(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::LONG){
		    	Long* h=(Long*)b;
		    	return new Double(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::DOUBLE){
		    	Double* h=(Double*)b;
		    	return new Double(h->getValue()-g->getValue());
		    }else{
		    	std::string s("-");
		    	errorMessage(&s,a,b);
		    }
		}else if(a->getType()==Object::LONG){
			Long* g=(Long*)a;
		    if(b->getType()==Object::SHORT){
		    	Short* h=(Short*)b;
		    	return new Long(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::INTEGER){
		    	Integer* h=(Integer*)b;
		    	return new Long(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::FLOAT){
		    	Float* h=(Float*)b;
		    	return new Double(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::LONG){
		    	Long* h=(Long*)b;
		    	return new Long(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::DOUBLE){
		    	Double* h=(Double*)b;
		    	return new Long(h->getValue()-g->getValue());
		    }else{
		    	std::string s("-");
		    	errorMessage(&s,a,b);
		    }
		}else if(a->getType()==Object::FLOAT){
			Float* g=(Float*)a;
		    if(b->getType()==Object::SHORT){
		    	Short* h=(Short*)b;
		    	return new Float(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::INTEGER){
		    	Integer* h=(Integer*)b;
		    	return new Float(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::FLOAT){
		    	Float* h=(Float*)b;
		    	return new Float(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::LONG){
		    	Long* h=(Long*)b;
		    	return new Double(h->getValue()-g->getValue());
		    }else if(b->getType()==Object::DOUBLE){
		    	Double* h=(Double*)b;
		    	return new Double(h->getValue()-g->getValue());
		    }else{
		    	std::string s("-");
		    	errorMessage(&s,a,b);
		    }
		}else{
	    	std::string s("-");
	    	errorMessage(&s,a,b);
	    }
		return NULL;
	}

	static int compareNumbers(Object* a,Object* b){
		if(a->getType()==Object::SHORT){
			Short* g=(Short*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return h->getValue()-g->getValue();
			}else{
				Double* h=(Double*)b;
				return h->getValue()-g->getValue();
			}
		}else if(a->getType()==Object::INTEGER){
			Integer* g=(Integer*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return h->getValue()-g->getValue();
			}else{
				Double* h=(Double*)b;
				return h->getValue()-g->getValue();
			}
		}else if(a->getType()==Object::DOUBLE){
			Double* g=(Double*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return h->getValue()-g->getValue();
			}else{
				Double* h=(Double*)b;
				return h->getValue()-g->getValue();
			}
		}else if(a->getType()==Object::LONG){
			Long* g=(Long*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return h->getValue()-g->getValue();
			}else{
				Double* h=(Double*)b;
				return h->getValue()-g->getValue();
			}
		}else if(a->getType()==Object::FLOAT){
			Float* g=(Float*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return h->getValue()-g->getValue();
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return h->getValue()-g->getValue();
			}else{
				Double* h=(Double*)b;
				return h->getValue()-g->getValue();
			}
		}
		return 0;
	}

	static bool isNumber(Object* n){
		return n->getType()>=Object::SHORT && n->getType()<=Object::DOUBLE;
	}

	static inline Object* arithmeticMultiplicationH(Object* a,Object* b){
		if(a->getType()==Object::SHORT){
			Short* g=(Short*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return new Short(h->getValue()*g->getValue());
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return new Integer(h->getValue()*g->getValue());
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return new Float(h->getValue()*g->getValue());
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return new Long(h->getValue()*g->getValue());
			}else if(b->getType()==Object::DOUBLE){
				Double* h=(Double*)b;
				return new Float(h->getValue()*g->getValue());
			}else{
				std::string s("*");
				errorMessage(&s,a,b);
			}
		}else if(a->getType()==Object::INTEGER){
			Integer* g=(Integer*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return new Integer(h->getValue()*g->getValue());
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return new Integer(h->getValue()*g->getValue());
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return new Float(h->getValue()*g->getValue());
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return new Long(h->getValue()*g->getValue());
			}else if(b->getType()==Object::DOUBLE){
				Double* h=(Double*)b;
				return new Double(h->getValue()*g->getValue());
			}else{
				std::string s("*");
				errorMessage(&s,a,b);
			}
		}else if(a->getType()==Object::DOUBLE){
			Double* g=(Double*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return new Double(h->getValue()*g->getValue());
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return new Double(h->getValue()*g->getValue());
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return new Double(h->getValue()*g->getValue());
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return new Double(h->getValue()*g->getValue());
			}else if(b->getType()==Object::DOUBLE){
				Double* h=(Double*)b;
				return new Double(h->getValue()*g->getValue());
			}else{
				std::string s("*");
				errorMessage(&s,a,b);
			}
		}else if(a->getType()==Object::LONG){
			Long* g=(Long*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return new Long(h->getValue()*g->getValue());
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return new Long(h->getValue()*g->getValue());
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return new Double(h->getValue()*g->getValue());
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return new Long(h->getValue()*g->getValue());
			}else if(b->getType()==Object::DOUBLE){
				Double* h=(Double*)b;
				return new Long(h->getValue()*g->getValue());
			}else{
				std::string s("*");
				errorMessage(&s,a,b);
			}
		}else if(a->getType()==Object::FLOAT){
			Float* g=(Float*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return new Float(h->getValue()*g->getValue());
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return new Float(h->getValue()*g->getValue());
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return new Float(h->getValue()*g->getValue());
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return new Double(h->getValue()*g->getValue());
			}else if(b->getType()==Object::DOUBLE){
				Double* h=(Double*)b;
				return new Double(h->getValue()*g->getValue());
			}else{
				std::string s("*");
				errorMessage(&s,a,b);
			}
		}else{
			std::string s("*");
			errorMessage(&s,a,b);
		}
		return NULL;
	}

	static inline Object* arithmeticDivisionH(Object* a,Object* b){
		if(a->getType()==Object::SHORT){
			Short* g=(Short*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return new Short(h->getValue()/g->getValue());
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return new Integer(h->getValue()/g->getValue());
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return new Float(h->getValue()/g->getValue());
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return new Long(h->getValue()/g->getValue());
			}else if(b->getType()==Object::DOUBLE){
				Double* h=(Double*)b;
				return new Float(h->getValue()/g->getValue());
			}else{
				std::string s("/");
				errorMessage(&s,a,b);
			}
		}else if(a->getType()==Object::INTEGER){
			Integer* g=(Integer*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return new Integer(h->getValue()/g->getValue());
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return new Integer(h->getValue()/g->getValue());
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return new Float(h->getValue()/g->getValue());
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return new Long(h->getValue()/g->getValue());
			}else if(b->getType()==Object::DOUBLE){
				Double* h=(Double*)b;
				return new Double(h->getValue()/g->getValue());
			}else{
				std::string s("/");
				errorMessage(&s,a,b);
			}
		}else if(a->getType()==Object::DOUBLE){
			Double* g=(Double*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return new Double(h->getValue()/g->getValue());
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return new Double(h->getValue()/g->getValue());
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return new Double(h->getValue()/g->getValue());
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return new Double(h->getValue()/g->getValue());
			}else if(b->getType()==Object::DOUBLE){
				Double* h=(Double*)b;
				return new Double(h->getValue()/g->getValue());
			}else{
				std::string s("/");
				errorMessage(&s,a,b);
			}
		}else if(a->getType()==Object::LONG){
			Long* g=(Long*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return new Long(h->getValue()/g->getValue());
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return new Long(h->getValue()/g->getValue());
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return new Double(h->getValue()/g->getValue());
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return new Long(h->getValue()/g->getValue());
			}else if(b->getType()==Object::DOUBLE){
				Double* h=(Double*)b;
				return new Long(h->getValue()/g->getValue());
			}else{
				std::string s("*");
				errorMessage(&s,a,b);
			}
		}else if(a->getType()==Object::FLOAT){
			Float* g=(Float*)a;
			if(b->getType()==Object::SHORT){
				Short* h=(Short*)b;
				return new Float(h->getValue()/g->getValue());
			}else if(b->getType()==Object::INTEGER){
				Integer* h=(Integer*)b;
				return new Float(h->getValue()/g->getValue());
			}else if(b->getType()==Object::FLOAT){
				Float* h=(Float*)b;
				return new Float(h->getValue()/g->getValue());
			}else if(b->getType()==Object::LONG){
				Long* h=(Long*)b;
				return new Double(h->getValue()/g->getValue());
			}else if(b->getType()==Object::DOUBLE){
				Double* h=(Double*)b;
				return new Double(h->getValue()/g->getValue());
			}else{
				std::string s("/");
				errorMessage(&s,a,b);
			}
		}else{
			std::string s("/");
			errorMessage(&s,a,b);
		}
		return NULL;
	}

public:
	static Object* arithmeticAddition(Object* a,Object* b){
		Object* res=arithmeticAdditionH(a,b);
		deleteObject(a);
		deleteObject(b);
		return res;
	}

	static Object* arithmeticSubtraction(Object* a,Object* b){
		Object* res=arithmeticSubtractionH(a,b);
		deleteObject(a);
		deleteObject(b);
		return res;
	}

	static Object* arithmeticDivision(Object* a,Object* b){
		Object* res=arithmeticDivisionH(a,b);
		deleteObject(a);
		deleteObject(b);
		return res;
	}

	static Object* arithmeticMultiplication(Object* a,Object* b){
		Object* res=arithmeticMultiplicationH(a,b);
		deleteObject(a);
		deleteObject(b);
		return res;
	}

	static void setPtr(Object *&f,Object* o){
		if(f!=NULL){
			f->ref--;
			if(f->ref<=0){
				delete f;
			}
		}
		f=o;
		if(f!=NULL){
			f->ref++;
		}
	}

	static Boolean* equal(Object* a,Object* b){
		Boolean *res=NULL;
		if(a==NULL && b==NULL){
			res=new Boolean(true);
		}else if(a==NULL || b==NULL){
			res=new Boolean(false);
		}else if(isNumber(a) && isNumber(b)){
			int r=compareNumbers(a,b);//b-a
			res=new Boolean(r==0);
		}else if(a->getType()==Object::BOOLEAN && b->getType()==Object::BOOLEAN){
			Boolean* b1=(Boolean*)a,*b2=(Boolean*)b;
			res=new Boolean(b1->getValue()==b2->getValue());
		}else if(a->getType()==Object::CHARACTER && b->getType()==Object::CHARACTER){
			Character* b1=(Character*)a,*b2=(Character*)b;
			res=new Boolean(b1->getValue()==b2->getValue());
		}else if(a->getType()==Object::STRING && b->getType()==Object::STRING){
			String* b1=(String*)a,*b2=(String*)b;
			res=new Boolean(b1->getValue()==b2->getValue());
		}else if(a->getType()==Object::POINTER && b->getType()==Object::POINTER){
			Pointer* b1=(Pointer*)a,*b2=(Pointer*)b;
			res=new Boolean(b1->getValue()==b2->getValue());
		}else{//user defined
			return new Boolean(a->equals(b));
		}
		deleteObject(a);
		deleteObject(b);
		return res;
	}

	static Boolean* notEqual(Object* a,Object* b){
		Boolean *res=NULL;
		if(a==NULL && b==NULL){
			res=new Boolean(false);
		}else if(a==NULL || b==NULL){
			res=new Boolean(true);
		}else if(isNumber(a) && isNumber(b)){
			int r=compareNumbers(a,b);//b-a
			res=new Boolean(r!=0);
		}else if(a->getType()==Object::BOOLEAN && b->getType()==Object::BOOLEAN){
			Boolean* b1=(Boolean*)a,*b2=(Boolean*)b;
			res=new Boolean(b1->getValue()!=b2->getValue());
		}else if(a->getType()==Object::CHARACTER && b->getType()==Object::CHARACTER){
			Character* b1=(Character*)a,*b2=(Character*)b;
			res=new Boolean(b1->getValue()!=b2->getValue());
		}else if(a->getType()==Object::STRING && b->getType()==Object::STRING){
			String* b1=(String*)a,*b2=(String*)b;
			res=new Boolean(b1->getValue().compare(b2->getValue())==0);
		}else if(a->getType()==Object::POINTER && b->getType()==Object::POINTER){
			Pointer* b1=(Pointer*)a,*b2=(Pointer*)b;
			res=new Boolean(b1->getValue()!=b2->getValue());
		}else{//user defined
			return new Boolean(!a->equals(b));
		}
		deleteObject(a);
		deleteObject(b);
		return res;
	}

	static Boolean* equalOrBigger(Object* a,Object* b){
		if(a==NULL && b==NULL){
            deleteObject(a);
			deleteObject(b);
			return new Boolean(true);
		}else if(a==NULL || b==NULL){
            deleteObject(a);
			deleteObject(b);
			return new Boolean(false);
		}
		if(a->getType()==Object::CHARACTER && b->getType()==Object::CHARACTER){
			Character* b1=(Character*)a,*b2=(Character*)b;
			Boolean *res=new Boolean(b1->getValue()>=b2->getValue());
			deleteObject(a);
			deleteObject(b);
			return res;
		}else if(isNumber(a) && isNumber(b)){
			int res=compareNumbers(a,b);
			deleteObject(a);
			deleteObject(b);
			return new Boolean(res<=0);
		}else{
			std::string s(">=");
			errorMessage(&s,a,b);
			deleteObject(a);
			deleteObject(b);
			return NULL;
		}
	}

	static Boolean* equalOrLesser(Object* a,Object* b){
		if(a==NULL && b==NULL){
            deleteObject(a);
			deleteObject(b);
			return new Boolean(true);
		}else if(a==NULL || b==NULL){
            deleteObject(a);
			deleteObject(b);
			return new Boolean(false);
		}
		if(a->getType()==Object::CHARACTER && b->getType()==Object::CHARACTER){
			Character* b1=(Character*)a,*b2=(Character*)b;
			Boolean *res=new Boolean(b1->getValue()<=b2->getValue());
			deleteObject(a);
			deleteObject(b);
			return res;
		}else if(isNumber(a) && isNumber(b)){
			int res=compareNumbers(a,b);
			deleteObject(a);
			deleteObject(b);
			return new Boolean(res>=0);
		}else{
			std::string s("<=");
			errorMessage(&s,a,b);
			deleteObject(a);
			deleteObject(b);
			return NULL;
		}
	}

	static Boolean* lesser(Object* a,Object* b){
        if(a==NULL && b==NULL){
            deleteObject(a);
			deleteObject(b);
			return new Boolean(false);
		}else if(a==NULL || b==NULL){
            deleteObject(a);
			deleteObject(b);
			return new Boolean(false);
		}
		if(a->getType()==Object::CHARACTER && b->getType()==Object::CHARACTER){
			Character* b1=(Character*)a,*b2=(Character*)b;
			Boolean *res=new Boolean(b1->getValue()<b2->getValue());
			deleteObject(a);
			deleteObject(b);
			return res;
		}else if(isNumber(a) && isNumber(b)){
			int res=compareNumbers(a,b);
			deleteObject(a);
			deleteObject(b);
			return new Boolean(res>0);
		}else{
			std::string s("<");
			errorMessage(&s,a,b);
			deleteObject(a);
			deleteObject(b);
			return NULL;
		}
	}

	static Boolean* bigger(Object* a,Object* b){
		if(a==NULL && b==NULL){
            deleteObject(a);
			deleteObject(b);
			return new Boolean(true);
		}else if(a==NULL || b==NULL){
            deleteObject(a);
			deleteObject(b);
			return new Boolean(false);
		}
		if(a->getType()==Object::CHARACTER &&  b->getType()==Object::CHARACTER){
			Character* b1=(Character*)a,*b2=(Character*)b;
			Boolean* res=new Boolean(b1->getValue()<b2->getValue());
			deleteObject(a);
			deleteObject(b);
			return res;
		}else if(isNumber(a) && isNumber(b)){
			int res=compareNumbers(a,b);
			deleteObject(a);
			deleteObject(b);
			return new Boolean(res<0);
		}else{
			std::string s(">");
			errorMessage(&s,a,b);
			deleteObject(a);
			deleteObject(b);
			return NULL;
		}
	}

	static bool isTrue(Object* a){
		if(a==NULL || a->getType()!=Object::BOOLEAN){
			deleteObject(a);
			return false;
		}else{
			Boolean *t=(Boolean*)a;
			bool res=t->getValue();
			deleteObject(a);
			return res;
		}
	}

	static Boolean* logicNo(Object* a){
		if(a==NULL){
			deleteObject(a);
			return NULL;
		}
		if(a->getType()==Object::BOOLEAN){
			Boolean* b1=(Boolean*)a,*res=new Boolean(!b1->getValue());
			deleteObject(a);
			return res;
		}else{
			std::cerr<<"Operator \"!\" not applicable to "<<a->getType()<<std::endl;
			deleteObject(a);
			return NULL;
		}
	}

	static Boolean* logicOr(Object* a,Object* b){
		if(a==NULL || b==NULL){
			deleteObject(a);
			deleteObject(b);
			return NULL;
		}
		if(a->getType()==Object::BOOLEAN && b->getType()==Object::BOOLEAN){
			Boolean* b1=(Boolean*)a,*b2=(Boolean*)b,*res=new Boolean(b1->getValue() || b2->getValue());
			deleteObject(a);
			deleteObject(b);
			return res;
		}else{
			std::string s("|");
			errorMessage(&s,a,b);
			deleteObject(a);
			deleteObject(b);
			return NULL;
		}
	}

	static Boolean* logicAnd(Object* a,Object* b){
		if(a==NULL || b==NULL){
			deleteObject(a);
			deleteObject(b);
			return NULL;
		}
		if(a->getType()==Object::BOOLEAN && b->getType()==Object::BOOLEAN){
			Boolean* b1=(Boolean*)a,*b2=(Boolean*)b,*res=new Boolean(b1->getValue() && b2->getValue());
			deleteObject(a);
			deleteObject(b);
			return res;
		}else{
			std::string s("&");
			errorMessage(&s,a,b);
			deleteObject(a);
			deleteObject(b);
			return NULL;
		}
	}
};
}

#endif /* OPERATIONS_H_ */
