#ifndef THREAD_H
#define THREAD_H

<namespace>{

class Thread;

struct THREAD{
	void *ptr;
	Thread* thread;
};
}

#if defined _WIN32 || defined WIN32 || defined _WIN64 || defined WIN64
#include<windows.h>
#else
#include<pthread.h>
#include<semaphore.h>
#include<time.h>
#endif
<namespace>{

class Thread{
public:
	static std::list<Thread*> THREAD_TO_DESTROY;

private:
	void*(*func)(void*);
#if defined _WIN32 || defined WIN32 || defined _WIN64 || defined WIN64
	HANDLE handle=NULL,sem;
#else
	unsigned long id;
	sem_t sem;
	timespec ts;
#endif
	static std::list<Thread*>::iterator tit;
	THREAD t;

public:
#if defined _WIN32 || defined WIN32 || defined _WIN64 || defined WIN64
	Thread(void*(*f)(void*),void *ptr):func(f),sem(CreateSemaphore(NULL,0,1,NULL)){
		t.thread=this;
		t.ptr=ptr;
	}
	~Thread(){
		CloseHandle(handle);
		CloseHandle(sem);
	}
#else
	Thread(void*(*f)(void*),void* ptr):func(f){
		t.thread=this;
		t.ptr=ptr;
		sem_init(&sem,0,0);
	}
	~Thread(){
		sem_close(&sem);
	}
#endif
#if defined _WIN32 || defined WIN32 || defined _WIN64 || defined WIN64
	void startThread(){
		handle=CreateThread(0,0,(LPTHREAD_START_ROUTINE)func,&t,0,NULL);
	}

	void waitEnd(){
		WaitForSingleObject(handle,INFINITE);
	}

	void threadSleep(long i){
		WaitForSingleObject(sem,i);
	}

	void interruptThread(){
		ReleaseSemaphore(sem,1,NULL);
	}
#else
	void startThread(){
		pthread_create(&id,NULL,func,&t);
	}

	void waitEnd(){
		pthread_join(id,NULL);
	}

	void threadSleep(long i){
		clock_gettime(CLOCK_REALTIME,&ts);
		int s=i/1000;
		ts.tv_sec+=s;
		ts.tv_nsec=(i-s*1000)*1000000;
		sem_timedwait(&sem,&ts);
	}

	void interruptThread(){
		sem_post(&sem);
	}
#endif
	static void deleteThreads(){
		Thread *as;
		if(THREAD_TO_DESTROY.size()<1){return;}
		for(tit=THREAD_TO_DESTROY.begin();tit!=THREAD_TO_DESTROY.end();){
			as=*tit;
			tit=THREAD_TO_DESTROY.erase(tit);
			as->waitEnd();
			delete as;
		}
	}
};
}

#endif//THREAD_H
